"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const https_1 = require("https");
const FormData = require("form-data");
const TRANSFORM_FIELDS = {
    replyTo: 'h:Reply-To'
};
const ADDRESS_KEYS = ['from', 'to', 'cc', 'bcc', 'replyTo'];
const CONTENT_KEYS = ['subject', 'text', 'html'];
class MailgunTransport {
    constructor(options) {
        this.name = 'MailgunTransport';
        this.version = 'N/A';
        this.requestConfig = {
            protocol: 'https:',
            hostname: options.hostname || 'api.mailgun.net',
            path: `/v3/${options.auth.domain}/messages`,
            auth: `api:${options.auth.apiKey}`
        };
    }
    combineTarget(target) {
        return target.name ? `${target.name} <${target.address}>` : target.address;
    }
    appendAddresses(form, data) {
        ADDRESS_KEYS.forEach(target => {
            if (!data[target])
                return;
            let value;
            if (Array.isArray(data[target])) {
                value = data[target].map(this.combineTarget).join(',');
            }
            else {
                value = this.combineTarget(data[target]);
            }
            form.append(TRANSFORM_FIELDS[target] || target, value);
        });
    }
    appendContent(form, data) {
        CONTENT_KEYS.forEach(content => {
            if (!data[content])
                return;
            form.append(TRANSFORM_FIELDS[content] || content, data[content]);
        });
    }
    appendAttachments(form, data) {
        if (!Array.isArray(data.attachments))
            return;
        data.attachments.forEach((attachment) => {
            let buffer = Buffer.from(attachment.content, attachment.encoding);
            form.append('inline', buffer, {
                filename: attachment.cid,
                contentType: attachment.contentType,
                knownLength: buffer.length
            });
        });
    }
    submitForm(form) {
        return new Promise((resolve, reject) => {
            let req = https_1.request(Object.assign({
                method: 'POST',
                headers: form.getHeaders()
            }, this.requestConfig));
            form.pipe(req);
            req.on('response', (res) => {
                let chunks = [];
                res.on('data', (chunk) => chunks.push(chunk));
                res.on('end', () => {
                    let answer = Buffer.concat(chunks).toString();
                    if (res.statusCode === 200) {
                        resolve(answer);
                    }
                    else {
                        reject(answer);
                    }
                });
                res.on('error', (error) => {
                    reject(error);
                });
            });
        });
    }
    send(mail, done) {
        setImmediate(() => {
            mail.normalize((error, data) => {
                if (error)
                    return done(error);
                let form = new FormData();
                this.appendAddresses(form, data);
                this.appendContent(form, data);
                this.appendAttachments(form, data);
                this.submitForm(form)
                    .then(() => {
                    done(null, {
                        envelope: mail.message.getEnvelope(),
                        messageId: mail.message.messageId(),
                        message: form
                    });
                })
                    .catch((e) => done(e));
            });
        });
    }
}
exports.MailgunTransport = MailgunTransport;
